;******************************************************
; ProcChr - decode host cmd and update disp
;******************************************************
ProcChr:

	tst		fnthl				; is font set to lower?
	breq	proc1				; yes, use the jmp table
	cpi		inpt, 0x80			; no, is it a lower 128 code?
	brcs	ASCII				; yes, just treat as standard char
proc1:
	ldi 	ZH, 0x05			; no, setup jump table page address
	mov		ZL, inpt			; put the input chr into Z
	ijmp						; jump table based on INPT

;========================================================
; Commands called from the Jump Table
;

SET_PRIM:
	ldi		inpt, 0x00
	mov		fntsel, inpt		; select primary font
	ret

SET_ALT:
	ldi		inpt, 0x08
	mov		fntsel, inpt		; select alternate font
	ret

SET_UPPER:
	ldi		inpt, 0x80
	mov		fnthl, inpt			; select upper 128 bytes of the font
	ret

SET_LOWER:
	ldi		inpt, 0x00
	mov		fnthl, inpt			; select lower 128 bytes of the font
	ret

TAB:
	ldi		inpt, 0x20			; convert tabs to white spaces
	st		Y+, inpt			; save space & advance
	inc		hpos				; 
	ldi		inpt, 0x07			; tabs are fixed  0,8,16,24,32, 40 (will wrap to next line if=40)	
	and		inpt, hpos			; mask out cursor column
	brne	TAB					; not to next tab, so do again
	rjmp	wrap				; did it wrap?

SPECIAL:
	andi	inpt, 0x1F			; mask out 0-31, and fall thru to ASCII code
ASCII:
	or		inpt, fnthl			; mask in the lower/upper select bit
	st		Y+, inpt			; store chr
	inc		hpos				; inc hpos
WRAP:
	cpi		hpos, 0x28			; past eol?
	brne	wrap1				; yes, continue
	ldi		hpos, 0x00			; reset hpos
	inc		vpos				; inc v pos
	cpi		vpos, 0x19			; is it below last line (25)?
	brne	wrap1				; no, do cursor and exit
	ldi		YL, 0x20			; yes, scroll up
	ldi		vpos, 0x18			; set to last line (24)
	rjmp	scrollup	
wrap1:
	ld		curschr, Y			; save chr under cursor
	ret							; return to Main

RETURN: 
	ldi		inpt, 0x20			; 
ret1:
	st		Y+, inpt			; 
	inc		hpos				; 
	cpi		hpos, 0x28			;
	brne	ret1				; clear to end of line
	ldi		hpos, 0x00			; set cursor to first column
	sbiw	YL, 0x28			; adjust Y to first char
	ld		curschr, Y			; save chr under cursor
	ret							; return to Main

LINEFEED:
	cpi		vpos, 0x18			; are we at line 24?
	breq	SCROLLUP			; yes, need to scroll up one line
	inc		vpos				; no, move cursor down 1 line
	adiw	YL, 0x28			; adjust Y pointer
	ld		curschr, Y			; save chr under cursor
	ret							; return to Main

DELETE:
	ldi		inpt, 0x20			; space
	st		Y, inpt				; save to SRAM
	mov		Curschr, inpt		; save chr under cursor (space)
	ret							; return to Main

BACKSPACE:
	cpi		hpos, 0x00			; are we already at the left edge?
	brne	del1				; no, move left one chr
	cpi		vpos, 0x00			; are we already on the top line?
	breq	del2  				; yes, nothing to do, so end
	dec		vpos				; no, move up one line
	ldi		hpos, 0x28			; & set the cursor to the right-most chr (+1 for next instruction)
del1:	
	dec		hpos				; move cursor left one chr
	ldi		inpt, 0x20			; space
	st		-Y, inpt			; remove last chr (destructive BS)
	mov		Curschr, inpt		; save chr under cursor (space)
del2:
	ret							; return to Main

CURSHOME:						; move cursor to Home w/o clear
	ldi		YH, 0x00			; 
	rjmp	formfeed1

FORMFEED:						; Clear the Screen
	ldi		ZL, 0x60
	ldi		ZH, 0x00			; set to start of SRAM
	ldi		YL, 0xE8
	ldi		YH,	0x03			; set Y to 1000
	ldi		inpt, 0x20			; " " space chracter
ffloop:
	st 		Z+, inpt			; save to SRAM			
	sbiw	YL, 0x01			; dec Y
	brne	ffloop				; do until Y=0
formfeed1:
	ldi		YL, 0x60			; set Y to first chr
	ldi		hpos, 0x00			; set cursor to top left pos
	ldi		vpos, 0x00			; 
	ld		curschr, Y			; save chr under cursor
	ret							; return to Main

SCROLLUP:
	push	YL
	push	YH
	ldi		YL, 0x60			; start of first line
	ldi		YH, 0x00			; 
	ldi		ZL, 0x88			; start of second line		
	ldi		ZH, 0x00			; 
scup1:
	ld		inpt, Z+			; get line 2 
	st		Y+, inpt			; place in line 1
	cpi		ZL, 0x48
	brne	scup1
	cpi		ZH, 0x04			; at the end of disp RAM?
	brne	scup1				; no, do again
	ldi		inpt, 0x20			; space
scup2:
	st		Y+, inpt			; fill last line with spaces
	cpi		YL, 0x48
	brne	scup2
	pop		YH
	pop		YL
	ld		curschr, Y			; save chr under cursor
	ret

SCROLLDN:
	push	YL
	push	YH
	ldi		YL, 0x20			; end of line 24
	ldi		YH, 0x04			; 
	ldi		ZL, 0x48			; end of line 25	
	ldi		ZH, 0x04			; 
scdn1:
	ld		inpt, -Y			; get line 2 
	st		-Z, inpt			; place in line 1
	cpi		YL, 0x60
	brne	scdn1
	cpi		YH, 0x00			; at the end of disp RAM?
	brne	scdn1				; no, do again
	ldi		inpt, 0x20			; space
scdn2:
	st		-Z, inpt			; fill last line with spaces
	cpi		ZL, 0x60
	brne	scdn2
	cpi		ZH, 0x00			; at the end of disp RAM?
	brne	scdn2				; no, do again
	pop		YH
	pop		YL
	ld		curschr, Y			; save chr under cursor
	ret

SCROLLLF:
	push	YL
	push	YH
	ldi		YL, 0x60			; start of first line
	ldi		YH, 0x00			; 
	ldi		ZL, 0x61			; start of second line		
	ldi		ZH, 0x00			; 
sclf1:
	ld		inpt, Z+			; get line 2 
	st		Y+, inpt			; place in line 1
	cpi		ZL, 0x48
	brne	sclf1
	cpi		ZH, 0x04			; at the end of disp RAM?
	brne	sclf1				; no, do again
	ldi		inpt, 0x20			; space
sclf2:
	st		Y, inpt				; fill last line with spaces
	sbiw	YL, 0x28				; 
	cpi		YL, 0x5F			;
	brne	sclf2				;
	cpi		YH, 0x00			; at the end of disp RAM?
	brne	sclf2				; no, do again
	pop		YH
	pop		YL
	ld		curschr, Y			; save chr under cursor
	ret

SCROLLRT:
	push	YL
	push	YH
	ldi		YL, 0x47			; end of line 24
	ldi		YH, 0x04			; 
	ldi		ZL, 0x48			; end of line 25	
	ldi		ZH, 0x04			; 
scrt1:
	ld		inpt, -Y			; get line 2 
	st		-Z, inpt			; place in line 1
	cpi		YL, 0x60
	brne	scrt1
	cpi		YH, 0x00			; at the end of disp RAM?
	brne	scrt1				; no, do again
	ldi		inpt, 0x20			; space
scrt2:
	st		Y, inpt				; fill left column with spaces
	adiw	YL, 0x28				; 
	cpi		YL, 0x48
	brne	scrt2
	cpi		YH, 0x04			; at the end of disp RAM?
	brne	scrt2				; no, do again
	pop		YH
	pop		YL
	ld		curschr, Y			; save chr under cursor
	ret

SETCOL:
	andi	inpt, 0x3F			; mask lower 6 bits
	sub		YL, hpos			;
	sbci	YH, 0x00			; 
	mov		hpos, inpt			; update hpos
	ldi		inpt, 0x00			;
	add		YL, hpos			;
	adc		YH, inpt			;	
	ld		curschr, Y			; save chr under cursor
	ret

SETROW:
	ldi		YL, 0x28			; # chrs per line
	ldi		YH, 0x60			; offset to start of SRAM 
	andi	inpt, 0x1F			; mask down to Line 0-24
	mov		vpos, inpt			; save new vertical position
	mul		YL, vpos			; multiply by chrs per line
	ldi		inpt, 0x00			; need a zero for upper byte addition
	add		R0, YH				; add SRAM start offset to result
	adc		R1, inpt			; add the carry bit
	movw	YL, R0				; mov it back to Y reg
	add		YL, hpos			; add in hpos
	adc		YH, inpt			; add the carry bit
	ld		curschr, Y			; save chr under cursor
	ret

Curs_us:
	ldi		Cursctl, 0x5F		; no, set cursor to underscore
	ret

Curs_blk:
	ldi		Cursctl, 0x0D		; set cursor to BLOCK
	ret

Curs_off:
	ldi		Cursctl, 0x00		;
	ret

Curs_blnk:
	ldi		Cursclk, 0x00
	ret

Curs_on:
	ldi		Cursclk, 0xff
	ret

Cursup:
	cpi		vpos, 0x00			; 
	breq	Null				; already at top, nothing to do
	dec		vpos				;
	sbiw	YL, 0x28			; 
	ld		curschr, Y			; save chr under cursor
	ret

CursDn:
	cpi		vpos, 0x18			; 
	breq	Null				; already at bottom, nothing to do
	inc		vpos				;
	adiw	YL, 0x28			; 
	ld		curschr, Y			; save chr under cursor
	ret

Curslf:
	cpi		hpos, 0x00			; 
	breq	Null				; already at top, nothing to do
	dec		hpos				;
	sbiw	YL, 0x01			; 
	ld		curschr, Y			; save chr under cursor
	ret

Cursrt:
	cpi		hpos, 0x27			; 
	breq	Null				; already at bottom, nothing to do
	inc		hpos				;
	adiw	YL, 0x01			; 
	ld		curschr, Y			; save chr under cursor
;   ret
NULL:
	ret

.cseg
.org 0x500
;
; jump table for each chr input from host
;

	rjmp	null				; 0
	rjmp	null				; 1
	rjmp	null				; 2
	rjmp	null				; 3
	rjmp	null				; 4
	rjmp	null				; 5
	rjmp	null				; 6
	rjmp	null				; 7
	rjmp	backspace			; 8
	rjmp	tab					; 9
	rjmp	linefeed			; A
	rjmp	null				; B
	rjmp	formfeed			; C
	rjmp	return				; D
	rjmp	null				; E
	rjmp	null				; F
	rjmp	null				; 10
	rjmp	null				; 1
	rjmp	null				; 2
	rjmp	null				; 3
	rjmp	null				; 4
	rjmp	null				; 5
	rjmp	null				; 6
	rjmp	null				; 7
	rjmp	null				; 8
	rjmp	null				; 9
	rjmp	null				; A
	rjmp	null				; B
	rjmp	null				; C
	rjmp	null				; D
	rjmp	null				; E
	rjmp	null				; F
	rjmp	ascii				; 20
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 30
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 40
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 50
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 60
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 70
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	delete				; 7F


	rjmp	special				; 80
	rjmp	special				; 1
	rjmp	special				; 2
	rjmp	special				; 3
	rjmp	special				; 4
	rjmp	special				; 5
	rjmp	special				; 6
	rjmp	special				; 7
	rjmp	special				; 8
	rjmp	special				; 9
	rjmp	special				; A
	rjmp	special				; B
	rjmp	special				; C
	rjmp	special				; D
	rjmp	special				; E
	rjmp	special				; F
	rjmp	special				; 90
	rjmp	special				; 1
	rjmp	special				; 2
	rjmp	special				; 3
	rjmp	special				; 4
	rjmp	special				; 5
	rjmp	special				; 6
	rjmp	special				; 7
	rjmp	special				; 8
	rjmp	special				; 9
	rjmp	special				; A
	rjmp	special				; B
	rjmp	special				; C
	rjmp	special				; D
	rjmp	special				; E
	rjmp	special				; F
	rjmp	setrow				; A0 row 0
	rjmp	setrow				; 1
	rjmp	setrow				; 2
	rjmp	setrow				; 3
	rjmp	setrow				; 4
	rjmp	setrow				; 5
	rjmp	setrow				; 6
	rjmp	setrow				; 7
	rjmp	setrow				; 8
	rjmp	setrow				; 9
	rjmp	setrow				; A
	rjmp	setrow				; B
	rjmp	setrow				; C
	rjmp	setrow				; D
	rjmp	setrow				; E
	rjmp	setrow				; F
	rjmp	setrow				; B0
	rjmp	setrow				; 1
	rjmp	setrow				; 2
	rjmp	setrow				; 3
	rjmp	setrow				; 4
	rjmp	setrow				; 5
	rjmp	setrow				; 6
	rjmp	setrow				; 7 
	rjmp	setrow				; 8 row 24
	rjmp	null				; 9
	rjmp	CursHome			; A
	rjmp	Curs_off			; B cursor off
	rjmp	Curs_blk			; C cursor block
	rjmp	Curs_us				; D cursor underscore
	rjmp	Curs_blnk			; E cursor blinking
	rjmp	Curs_on				; F cursor solid
	rjmp	setcol				; C0 - Col 0
	rjmp	setcol				; 1
	rjmp	setcol				; 2
	rjmp	setcol				; 3
	rjmp	setcol				; 4
	rjmp	setcol				; 5
	rjmp	setcol				; 6
	rjmp	setcol				; 7
	rjmp	setcol				; 8
	rjmp	setcol				; 9
	rjmp	setcol				; A
	rjmp	setcol				; B
	rjmp	setcol				; C
	rjmp	setcol				; D
	rjmp	setcol				; E
	rjmp	setcol				; F
	rjmp	setcol				; D0 - 16
	rjmp	setcol				; 1
	rjmp	setcol				; 2
	rjmp	setcol				; 3
	rjmp	setcol				; 4
	rjmp	setcol				; 5
	rjmp	setcol				; 6
	rjmp	setcol				; 7
	rjmp	setcol				; 8
	rjmp	setcol				; 9
	rjmp	setcol				; A
	rjmp	setcol				; B
	rjmp	setcol				; C
	rjmp	setcol				; D
	rjmp	setcol				; E
	rjmp	setcol				; F
	rjmp	setcol				; E0 - 32
	rjmp	setcol				; 1
	rjmp	setcol				; 2
	rjmp	setcol				; 3
	rjmp	setcol				; 4
	rjmp	setcol				; 5
	rjmp	setcol				; 6
	rjmp	setcol				; 7  - 39
	rjmp	CursUp				; 8
	rjmp	CursDn				; 9
	rjmp	CursLf				; A
	rjmp	CursRt				; B
	rjmp	scrollup			; C
	rjmp	scrolldn			; D
	rjmp	scrolllf			; E
	rjmp	scrollrt			; F
	rjmp	Set_Lower			; F0
	rjmp	Set_Upper			; 1
	rjmp	Set_prim			; 2
	rjmp	Set_alt				; 3
	rjmp	null				; 4
	rjmp	null				; 5
	rjmp	null				; 6
	rjmp	null				; 7
	rjmp	null				; 8
	rjmp	null				; 9
	rjmp	null				; A
	rjmp	null				; B
	rjmp	null				; C
	rjmp	null				; D
	rjmp	null				; E
	rjmp	null				; FF
