;******************************************************************************
; LCD Video Generation Code (16MHZ System Clock)
;******************************************************************************

; on entry, CP is high, LP is low
; FLM passes thru
;
; vline counts from 0 to 239 for each line and recycles
; hcnt is used to count the characters per line 0-39
; 
	movw	svz, ZL					; save Z reg contents
	movw	svacc, Acc				; save Z reg contents

	cpi		vline, 0xF0				; line 240 = last line
	brne	L200
	ldi		XL, 0x60				; first line
	ldi		XH, 0x00				; init SRAM pointer
	sbi		portc, flm				; 2	
	ldi		vline, 0x00				;	
	inc     Cursclk					; advance the cursor clock once per frame
	rjmp	DISP					;

L200:
	cpi		vline, 0xC8				; >= line 200?
	brcc	B1						

DISP:
	; set the SRAM character pointer (X) to the start of the current line (chrln)
	ldi		XL, 0x28				; # chrs per line
	ldi		XH, 0x60				; offset to start of SRAM 
	mov		J, vline				; Get current display line
	lsr		J						; divide by 8 
	lsr		J						; 
	lsr		J						; 
	mul		XL, J					; multiply by chrs per line
	ldi		J, 0x00					; 
	add		R0, XH					; add SRAM start offset to result
	adc		R1, J					; 
	movw	XL, R0					; mov it to X reg
									; set Z to point to base of font row
									; ASCII code will be added for each character
	mov 	ZH, vline				; ZH= current display line 
	andi	ZH, 0x07				; do ZH MOD 8
	ori		ZH, 0x10				; add $1000 for offset to start of font table
	or		ZH, fntsel				; adjust for primary/alt font
	ldi		hcnt, 0x27			; line counter -1 (39)
N1:
	ld 		ZL, X+				; 2   move curr chr in ZL
	lpm 	J, Z				; 3   get font byte for current chr on curr line
	swap	J					; 1   swap nibbles for high nibble first
	out		portb, J			; 1
	cbi		portc, clp			; 2
	swap	J					; 1   swap nibbles for high nibble first
	sbi		portc, clp			; 2
	out		portb, J			; 1
	cbi		portc, clp			; 2
	dec		hcnt				; 1
	sbi		portc, clp			; 2
	brne	N1					; 1/2

	ld 		ZL, X+				; 2   move curr chr in ZL
	lpm 	J, Z				; 3   get font byte for current chr on curr line
	swap	J					; 1   swap nibbles for high nibble first
	out		portb, J			; 1
	cbi		portc, clp			; 2
	swap	J					; 1   swap nibbles for high nibble first
	sbi		portc, clp			; 2
	sbi		portc, ldp			; 2
	out		portb, J			; 1
	cbi		portc, clp			; 2
	cbi		portc, ldp			; 2
	sbi		portc, clp			; 2
	rjmp	EOL					; 2

B1:									; blank line
	ldi		J, 0x00					; clr the display data
	out		portb, J
	ldi		J, 0x4F					; # of nibbles/line - 1
Lp1:
	cbi		portc, clp
	dec		J
	sbi		portc, clp
	brne	Lp1
	sbi		portc, ldp
	cbi		portc, clp
	cbi		portc, ldp
	sbi		portc, clp

EOL:								; end of line, we're done
	movw	ZL, svz					; restore Z register
	movw	Acc, svacc				; restore Acc Register
	inc		vline					; inc vertical line counter
	cbi		portc, flm				; yes, drop FLM	
	
	; RETI is in main program
